import { Component, OnInit } from '@angular/core';
import { FormControl } from '@angular/forms';
import { MatDialog } from '@angular/material';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from '../../../core/common/base/base.component';
import { AppRole } from '../../../core/common/enum/app-role';
import { SearchBarItem } from '../../../core/model/dashboard/search-bar-item';
import { ProjectCriteria } from '../../../core/query/project/project-criteria';
import { RequestItem } from '../../../core/query/request-item';
import { AuthService } from '../../../core/services/auth/auth.service';
import { ProgressIndicationService } from '../../../core/services/progress-indication/progress-indication-service';
import { ProjectService } from '../../../core/services/project/project.service';
import { SearchBarService } from '../../../core/services/search-bar/search-bar.service';
import { SingleAutoCompleteConfiguration } from '../../../library/auto-complete/single/single-auto-complete-configuration';
import { LanguageResolverService } from '../../../services/language-resolver/language-resolver.service';
import { UserDialogComponent } from './user-dialog/user-dialog.component';

export enum SearchBarType {
	Dataset = 0,
	Dmp = 1,
	Project = 2
}

@Component({
	selector: 'app-navigation',
	templateUrl: 'navigation.component.html',
	styleUrls: ['./navigation.component.scss']
})
export class NavigationComponent extends BaseComponent implements OnInit {
	progressIndication = false;
	public search = false;
	projectAutoCompleteConfiguration: SingleAutoCompleteConfiguration;
	public searchControl = new FormControl();
	filteredOptions: Observable<SearchBarItem[]>;

	events: string[] = [];
	opened: boolean;
	constructor(
		private authentication: AuthService,
		private dialog: MatDialog,
		private projectService: ProjectService,
		private searchBarService: SearchBarService,
		private router: Router,
		private progressIndicationService: ProgressIndicationService,
		public languageResolver: LanguageResolverService
	) {
		super();
	}

	ngOnInit() {
		this.progressIndicationService.getProgressIndicationObservable().pipe(takeUntil(this._destroyed)).subscribe(x => {
			setTimeout(() => { this.progressIndication = x; });
		});

		this.projectAutoCompleteConfiguration = {
			filterFn: this.searchProject.bind(this),
			initialItems: (extraData) => this.searchProject(''),
			displayFn: (item) => item['label'],
			titleFn: (item) => item['label']
		};

		this.filteredOptions = this.searchControl.valueChanges.debounceTime(500).distinctUntilChanged().flatMap(x => {
			return this.searchBarService.search(x);
		});
	}

	searchProject(query: string) {
		const projectRequestItem: RequestItem<ProjectCriteria> = new RequestItem();
		projectRequestItem.criteria = new ProjectCriteria();
		projectRequestItem.criteria.like = query;
		return this.projectService.getWithExternal(projectRequestItem);
	}

	public logout(): void {
		this.authentication.logout();
	}

	public isAuthenticated(): boolean {
		return !(!this.authentication.current());
	}

	public isAdmin(): boolean {
		if (!this.authentication.current()) { return false; }
		const principalRoles = this.authentication.current().authorities;
		for (let i = 0; i < principalRoles.length; i++) {
			if (principalRoles[i] === AppRole.Admin) {
				return true;
			}
		}
		return false;
	}

	public principalHasAvatar(): boolean {
		return this.authentication.current().avatarUrl != null;
	}

	public getPrincipalAvatar(): string {
		return this.authentication.current().avatarUrl;
	}

	openProfile() {
		const dialogRef = this.dialog.open(UserDialogComponent, {
			hasBackdrop: true,
			autoFocus: false,
			closeOnNavigation: true,
			disableClose: false,
			position: { top: '64px', right: '1em' }
		});
	}

	onOptionSelected(event: any) {
		this.search = false;
		this.searchControl.patchValue(null);
		const selectedSearchBarItem = event.option.value;
		if (selectedSearchBarItem.type === SearchBarType.Dataset) { this.router.navigate(['datasets/edit/' + selectedSearchBarItem.id]); }
		if (selectedSearchBarItem.type === SearchBarType.Project) { this.router.navigate(['projects/edit/' + selectedSearchBarItem.id]); }
		if (selectedSearchBarItem.type === SearchBarType.Dmp) { this.router.navigate(['plans/edit/' + selectedSearchBarItem.id]); }
	}

	transformType(type) {
		switch (type) {
			case SearchBarType.Dataset: return 'Dataset';
			case SearchBarType.Dmp: return 'DMP';
			case SearchBarType.Project: return 'Project';
		}
	}
}
