import { DataSource } from '@angular/cdk/table';
import { Component, OnInit, ViewChild } from '@angular/core';
import { MatPaginator, MatSnackBar, MatSort } from '@angular/material';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { ProjectStateType } from '../../../core/common/enum/project-state-type';
import { ProjectListingModel } from '../../../core/model/project/project-listing';
import { ProjectCriteria } from '../../../core/query/project/project-criteria';
import { ProjectService } from '../../../core/services/project/project.service';
import { DataTableRequest } from '../../../core/model/data-table/data-table-request';
import { LanguageResolverService } from '../../../services/language-resolver/language-resolver.service';
import { BreadcrumbItem } from '../../misc/breadcrumb/definition/breadcrumb-item';
import { IBreadCrumbComponent } from '../../misc/breadcrumb/definition/IBreadCrumbComponent';
import { ProjectCriteriaComponent } from './criteria/project-criteria.component';

@Component({
	selector: 'app-project-listing-component',
	templateUrl: 'project-listing.component.html',
	styleUrls: ['./project-listing.component.scss']
})
export class ProjectListingComponent implements OnInit, IBreadCrumbComponent {

	public breadCrumbs: Observable<BreadcrumbItem[]> = Observable.of([]);

	@ViewChild(MatPaginator) _paginator: MatPaginator;
	@ViewChild(MatSort) sort: MatSort;
	@ViewChild(ProjectCriteriaComponent) criteria: ProjectCriteriaComponent;

	host = environment.Server;
	dataSource: ProjectDataSource | null;
	displayedColumns: String[] = ['avatar', 'name', 'abbreviation', 'start', 'end', 'dmps'];

	constructor(
		public projectService: ProjectService,
		public router: Router,
		public languageService: TranslateService,
		public snackBar: MatSnackBar,
		public languageResolverService: LanguageResolverService
	) {

	}

	ngOnInit() {
		this.breadCrumbs = Observable.of([]);

		this.criteria.setCriteria(this.getDefaultCriteria());
		this.refresh();
		this.criteria.setRefreshCallback(() => this.refresh());
	}

	refresh() {
		this.dataSource = new ProjectDataSource(this.projectService, this._paginator, this.sort, this.languageService, this.snackBar, this.criteria);
	}

	rowClick(rowId: String) {
		this.router.navigate(['/projects/edit/' + rowId]);
	}

	getDefaultCriteria(): ProjectCriteria {
		const defaultCriteria = new ProjectCriteria();
		defaultCriteria.projectStateType = ProjectStateType.OnGoing;
		return defaultCriteria;
	}
}

export class ProjectDataSource extends DataSource<ProjectListingModel> {

	totalCount = 0;

	constructor(
		private _service: ProjectService,
		private _paginator: MatPaginator,
		private _sort: MatSort,
		private _languageService: TranslateService,
		private _snackBar: MatSnackBar,
		private _criteria: ProjectCriteriaComponent
	) {
		super();

		//this._paginator.page.pipe(takeUntil(this._destroyed)).subscribe((pageEvent: PageEvent) => {
		//    this.store.dispatch(new LoadPhotosRequestAction(pageEvent.pageIndex, pageEvent.pageSize))
		//})
	}

	connect(): Observable<ProjectListingModel[]> {
		const displayDataChanges = [
			this._paginator.page
			//this._sort.matSortChange
		];

		// If the user changes the sort order, reset back to the first page.
		//this._sort.matSortChange.pipe(takeUntil(this._destroyed)).subscribe(() => {
		//    this._paginator.pageIndex = 0;
		//})

		return Observable.merge(...displayDataChanges)
			.startWith(null)
			.switchMap(() => {
				const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
				let fields: Array<string> = new Array();
				if (this._sort.active) { fields = this._sort.direction === 'asc' ? ['+' + this._sort.active] : ['-' + this._sort.active]; }
				const request = new DataTableRequest<ProjectCriteria>(startIndex, this._paginator.pageSize, { fields: fields });
				request.criteria = this._criteria.criteria;
				return this._service.getPaged(request, "listing");
			})
			/*.catch((error: any) => {
			  this._snackBar.openFromComponent(SnackBarNotificationComponent, {
				data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
				duration: 3000,
				extraClasses: ['snackbar-warning']
			  });
			  this._criteria.onCallbackError(error.error);
			  return Observable.of(null);
			})*/
			.map(result => {
				result.data = result.data;
				return result;
			})
			.map(result => {
				return result;
			})
			.map(result => {
				if (!result) { return []; }
				if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
				return result.data.map(item => {
					item.dmps.map(dmp => {
						dmp.url = 'plans/edit/' + dmp.url;
						dmp.all = 'plans/project/' + item.id;
						return dmp;
					});
					return item;
				});
			});

	}

	disconnect() {
		// No-op
	}
}
